@@ -28,10 +28,12 @@
  */
 
 #include "iemguts.h"
+#include "canvasposition.h"
 
 #include "g_canvas.h"
 #include "m_imp.h"
 
+
 /* ------------------------- canvasposition ---------------------------- */
 
 static t_class *canvasposition_class;
@@ -47,58 +49,16 @@ typedef struct _canvasposition
 
 static void canvasposition_bang(t_canvasposition *x)
 {
-  t_canvas*c=x->x_canvas;
-  t_canvas*c0=0;
-  t_float x0=0., y0=0., width=0., height=0.;
-  int x1, y1, x2, y2;
-  t_float zoom = 1.;
-  t_atom alist[2];
-
-  if(!c) return;
-  x1 = x2 = y1 = y2 = 0;
-
-  if((c0=c->gl_owner)) {
-    width= (c0->gl_screenx2 - c0->gl_screenx1);
-    height=(c0->gl_screeny2 - c0->gl_screeny1);
-
-#if (defined PD_MAJOR_VERSION && defined PD_MINOR_VERSION) && (PD_MAJOR_VERSION > 0 || PD_MINOR_VERSION >= 47)
-    if(iemguts_check_atleast_pdversion(0,47,0)) {
-      zoom = c0->gl_zoom;
-    }
-#endif
-
-    gobj_getrect(&c->gl_obj.te_g, c0, &x1, &y1, &x2, &y2);
-  }
-  x0=c->gl_obj.te_xpix;
-  y0=c->gl_obj.te_ypix;
-
-
-  if(c0) {
-    SETFLOAT(alist+0, x2-x1);
-    SETFLOAT(alist+1, y2-y1);
-    outlet_anything(x->extraoutlet, gensym("size"), 2, alist);
-  }
-
-  SETFLOAT(alist+0, zoom);
-  outlet_anything(x->extraoutlet, gensym("zoom"), 1, alist);
-
-  if(c0) {
-    SETFLOAT(alist+0, width);
-    SETFLOAT(alist+1, height);
-    outlet_list(x->sizeoutlet, 0, 2, alist);
-  }
-
-  SETFLOAT(alist+0, x0/zoom);
-  SETFLOAT(alist+1, y0/zoom);
-  outlet_list(x->posoutlet, 0, 2, alist);
+  if(x->x_canvas)
+    canvasposition_get(x->posoutlet, x->sizeoutlet, x->extraoutlet,
+                       x->x_canvas->gl_owner, &x->x_canvas->gl_obj);
 }
 
 static void canvasposition_list(t_canvasposition *x, t_symbol*s, int argc, t_atom*argv)
 {
   t_canvas*c=x->x_canvas;
   t_canvas*c0=0;
-  int dx, dy;
-  t_float zoom = 1.;
+  t_float newX, newY;
   (void)s;
 
   if(!c) return;
@@ -113,24 +73,9 @@ static void canvasposition_list(t_canvasposition *x, t_symbol*s, int argc, t_ato
     pd_error(x, "expected <x> <y> as new position");
     return;
   }
-#if (defined PD_MAJOR_VERSION && defined PD_MINOR_VERSION) && (PD_MAJOR_VERSION > 0 || PD_MINOR_VERSION >= 47)
-  if(c0 && iemguts_check_atleast_pdversion(0,47,0)) {
-    zoom = c0->gl_zoom;
-  }
-#endif
-
-  dx = atom_getfloat(argv+0)*zoom - c->gl_obj.te_xpix;
-  dy = atom_getfloat(argv+1)*zoom - c->gl_obj.te_ypix;
-
-  if ((dx!=0)||(dy!=0)) {
-    if(c0&&glist_isvisible(c0))  {
-      gobj_displace((t_gobj*)c, c0, dx, dy);
-      canvas_fixlinesfor(c0, (t_text*)c);
-    } else {
-      c->gl_obj.te_xpix+=dx;
-      c->gl_obj.te_ypix+=dy;
-    }
-  }
+  newX = atom_getfloat(argv+0);
+  newY = atom_getfloat(argv+1);
+  canvasposition_set(c0, &c->gl_obj, newX, newY);
 }
 
 static void canvasposition_free(t_canvasposition *x)
